  '                         CHESS-2-PICO
  ' terminal program for external chess engine running on 2nd pico running TSCP
  ' TSCP uses COM1 port GP0,GP1 , terminal uses COM2 port GP4,GP5
  '
  ' Versions
  ' 0.1         colorchess_b27 auto mode only running on V050707b27
  ' 0.2         tiles back to 8x12 to potentially colour text
  ' 0.3         UI for player=white
  ' 0.4         focus on string handling and validation
  ' 0.5         parse all commands
  ' 0.6         plays a good match to the end
  ' 0.7         lists moves
  ' 0.8         Color list
  ' 0.9         Fixes Peters bugs


  debug=0       ' development help
  reply=0       ' 0=single player. 1=auto mode
  hlt=0         ' halt is the end of the game
  wtime=10     ' waiting time to get response from chess engine


  ' open COM port to second pico
  SetPin gp1,gp0,COM1
  Open "COM1:115200,2048" As #1   '115200 baud = 0.1ms per character
  If debug Then Open "a:/log.txt" For output As #3

  ' defines
  Dim respons$(30)  'max response from TSCP, look ahead so far plus board
  MODE 1:CLS        'screen definition
  score_template



  'a clean game
  Pause 1000
  restart_chess     'does not work when game has ended ?
  board
  Pause 1000

  'interactive terminal function
  Do

    ' bypass player when reply is automatic
    If reply=0 Then

      'make sure computer does not play
      Print #1,"off"
      If debug Then Print "off":Print #3,"off"
      Pause wtime
      n=get_response%()

      'read the next payer move
      Print @(260,440);
      Input "player move ";a$
      Print

    Else

      a$="on"   'make computer move for player as well

    End If

    'check validity - to be implemented
    valide = vali%(a$)

    If valide=4 Then  'it is a valid move

      Print #1,a$
      If debug Then Print a$:Print #3,a$
      Pause wtime
      n=get_response%()
      Inc hlt,game_end(n)
      game_flow(n)

      'show the board
      board

      'force the computer to play
      Print #1,"on"
      If debug Then Print "on":Print #3,"on"
      Pause wtime
      n=get_response%()
      Inc hlt,game_end(n)
      game_flow(n)

      'print respons$(0) 'the computer move
      If debug Then
        For i=0 To n
          Print respons$(i)
        Next
      End If

    Else 'this may be a command

      a$=cmd$(a$) 'convert to valid commands

      If Len(a$) Then
        Print #1,a$
        If debug Then Print a$:Print #3,a$
        Pause wtime
        n=get_response%()
        Inc hlt,game_end(n)
        game_flow(n)
      End If

    End If

    'show the board
    board

  Loop Until a$=Chr$(27) Or hlt>0
End



Sub game_flow(n)
  If n>0 Then
    If Instr(respons$(n-1),"Computer") Then
      Print @(xlist,ylist);Right$(respons$(n-1),4);
      If xlist<120 Then xlist = 100 - xlist Else xlist = 324-xlist
      Inc ylist,MM.Info(FONTHEIGHT)*((xlist=30)+(xlist=142))
      If ylist > ymaxlist Then xlist=142:ylist=28
    End If
  EndIf
End Sub

Sub score_template

  Box 1,1,221,478
  Print @(16,12);"WHITE BLACK   WHITE BLACK"
  For i=1 To 37
    Print @(4,16+i*MM.Info(FONTHEIGHT));Str$(i,2);"            ";Str$(i+37,2);
  Next

  'tiles
  TILE 0,0,RGB(black),RGB(white),28,40
  TILE 3,2,RGB(blue),RGB(white),10,38
  TILE 17,2,RGB(blue),RGB(white),10,38
  TILE 8,1,RGB(WHITE),RGB(BLACK),5,1
  TILE 22,1,RGB(WHITE),RGB(BLACK),5,1

  'game list related
  xlist=30:  ylist=28:  ymaxlist = 460

End Sub

  'detect computer says end of game
Function game_end(n)
  Local i,a$

  game_end=0
  For i=0 To 30
    a$ = respons$(i)
    Inc game_end,Instr(a$,"legal")    'no legal moves
    Inc game_end,Instr(a$,"1/2-1/2")  '50 moves
    Inc game_end,Instr(a$,"mates}")   'check mate
  Next

End Function


Sub board

  'print the board as ascii
  Print #1,"d"
  If debug Then Print "d":Print #3,"d"
  Pause wtime
  n=get_response%()

  'and display the board in graphics if enough in buffer.
  If Not debug And n>11 Then show_board

End Sub

Function vali%(aaa$) As integer

  vali%=0 'this is not a valid chess move
  Local a,b,c,d, aa$

  aa$=UCase$(aaa$)
  If Len(aa$)=4 Then
    a=Asc(Left$(aa$,1))
    b=Asc(Mid$(aa$,2,1))
    c=Asc(Mid$(aa$,3,1))
    d=Asc(Right$(aa$,1))
    If a>64 And a<73 Then Inc vali% 'a...h
    If c>64 And c<73 Then Inc vali% 'a...h
    If b>48 And b<57 Then Inc vali% '1...8
    If d>48 And d<57 Then Inc vali% '1...8
  End If

End Function

Function cmd$(a$)
  Local b$=LCase$(a$),tm

  cmd$=""
  If Instr(b$,"auto") Then cmd$="on":reply=1:Exit Function
  If Instr(b$,"xboard") Then cmd$="xboard":Exit Function
  If Instr(b$,"new") Then cmd$="new":Exit Function
  If Instr(b$,"bye") Then cmd$=Chr$(27):Exit Function
  If Instr(b$,"on") Then cmd$="on":Exit Function
  If Instr(b$,"help") Then cmd$="help":Exit Function
  If Instr(b$,"undo") Then cmd$="undo":Exit Function
  If Instr(b$,"bench") Then cmd$="bench":Exit Function
  If Instr(b$,"?") Then cmd$="help":Exit Function
  If Instr(b$,"st ") Then
    tm=Val(Right$(b$,Len(b$)-3))
    cmd$="st "+Str$(tm)
'    Exit Function
  End If
  If Instr(b$,"sd ") Then
    tm=Val(Right$(b$,Len(b$)-3))
    cmd$="sd "+Str$(tm)
'    Exit Function
  End If
End Function


Sub show_board
  If Not debug Then
    Local i%,j%,px%,py%,tx%,ty%,pc%,tc%,dx%,dy%
    Local fi$="Tiles\\b", ex$="w.bin", tl$, p$
    For i%=1 To 8  'first string is " ", then follows the board on strings 1-8
'      Print respons$(i%)
      py%=(i%-1)*48+12: ty%=i%-1
      For j%=4 To 18 Step 2
        px%=240+(j%-4)*24:tx%=(j%-4)/2

        'get piece
        p$=Mid$(respons$(i%),j%,1)

        'decide piece color (capitals are black pieces, lowercase white)
        If Asc(p$) < Asc("a") Then
          pc%=RGB(white)
        Else
          pc%=RGB(red)'(black)
        EndIf

        'decide tile color
        If (i%+j%/2)Mod 2 Then
          tc%=RGB(green)  'dark tiles
        Else
          tc%=RGB(myrtle) 'light tiles
        EndIf

        'get icon from SD card
        p$=fi$+LCase$(p$)+ex$           'for A: drive
        Open p$ For input As #2: tl$=Input$(200,#2):Close #2

        'draw icon on enlarged tile
        Box px%,py%,48,48,1,RGB(white),RGB(white) 'make the tiles virtually bigger
        GUI bitmap 4+px%,4+py%,tl$,40,40          'print the 40x40 tile in centre

        For dx%=0 To 5
          For dy%=0 To 3
            TILE 30+dx%+6*tx%, 1+dy%+4*ty%,tc%,pc%
          Next
        Next

      Next j%
    Next i%

    'board legend
    For k%=1 To 8
      Text 230,48*k%-16,Str$(9-k%)',"LT",,,rgb(white),rgb(black)
      Text 630,48*k%-16,Str$(9-k%)',"LT",,,rgb(white),rgb(black)
      Text 210+48*k%,0,Chr$(k%+64)',"LT",,,rgb(white),rgb(black)
      Text 210+48*k%,400,Chr$(k%+64)',"LT",,,rgb(white),rgb(black)
    Next k%
  End If
End Sub


  ' read response from chess module in maximum 20 strings
Function get_response%()
  Local n%=0, i%
  Local a$="",b$=""

  'debug clean buffer
  For i%=0 To 30
    respons$(i%)=""
  Next
  'debug

'  Timer =0
  Do
    a$=Input$(1,1)
    If a$=Chr$(13) Then
      respons$(n%)=b$
      b$=""
      Inc n%
    ElseIf a$=Chr$(10) Then
      'do nothing
    Else
      b$=b$+a$
    EndIf
'    If Timer>10000 Then b$=b$+"tscp>"    'sometimes TSCP does not respons
  Loop Until Instr(b$,"tscp>")
  respons$(n%)=b$                        'this is the tscp> prompt
  get_response%=n%

  If debug Then
    For i%=0 To n%
      Print respons$(i%)
    Next
  End If

End Function

  'restart chess module with a clean game
Sub restart_chess

  'flush UART buffer
  Do
    a$=Input$(1,#1)
  Loop While Loc(1)

  'send initial command that will fail becuase no pullup on RXD
  Print #1,"new"
  If debug Then Print "new":Print #3,"new"
  Pause wtime
  n=get_response%() 'read until prompt

  'send "new" to make the chess module perform a watchdog reset
  Print #1,"new"
  If debug Then Print "new":Print #3,"new"
  Pause wtime
  n=get_response%() 'read until prompt

End Sub
